import config
import images
import particle
import pyglet
import client
from vector import Vector, Vector3
import background
import player
from character import Character
from pyglet.window import key
from director import Director
from rect import Rect

SPEECH_AREA_CENTRE_X = 124
SPEECH_AREA_CENTRE_Y = 77
SPEECH_AREA_WIDTH = 180
SPEECH_AREA_HEIGHT = 100

USE_ORDERED_GROUPS = False
MAX_ORDERED_GROUPS = 13

def compare_depth (character1, character2):
  return (int)(character2.pos.z - character1.pos.z)

class Fight_client (client.Client):
  def __init__ (self, window):
    self.window = window
    self.world_size = Vector3 ( 2400, self.window.height , 90)
    self.background = background.Background(Vector (1000,self.window.height))
    
    self.fence_height = 150
    self.left_fence = Rect(0,0,10,self.fence_height)
    self.right_fence = Rect(self.world_size.x-10,0,self.world_size.x,self.fence_height)
        
    self.player_healthbar = Rect(20, self.window.height - 30, 120, self.window.height - 20)
    self.enemy_healthbar = Rect(self.window.width - 120, self.window.height - 30, self.window.width - 20, self.window.height - 20)
    
    self.key_handler = window.key_handler
    self.message_time_out = 0

    pyglet.clock.schedule(self.tick)
    
    self.fade_time = 2.0
    self.fading_rects = {}
    
    self.director = Director(self)

    self.paused = False

    self.start_up()

    self.message_sprite = pyglet.sprite.Sprite(images.speech_bubble)
    
    self.game_over_message = pyglet.text.Label("Game Over",x=self.window.width / 2,y=self.window.height / 2, font_size=36,
anchor_x="center", anchor_y="center", color = (255,0,0,255),halign="center")
    self.victory_message = pyglet.text.Label("Yoo is Winnar!",x=self.window.width / 2,y=self.window.height / 2, font_size=36,
anchor_x="center", anchor_y="center", color = (255,0,0,255),halign="center")
    #self.victory_message = pyglet.sprite.Sprite(pyglet.image.load ("data/winner.png"),x=self.window.width/2-300,y=50)
    self.restart_message = pyglet.text.Label("Press ENTER to restart",x=self.window.width / 2,y=self.window.height / 2 - 60, font_size=12,
anchor_x="center", anchor_y="center", color = (255,0,0,255),halign="center")

  def start_up(self):
    self.characters = []
    self.objects = []
    self.object_sprites = []
    self.screen_pos = Vector(0,0)
    self.object_sprite_batch = pyglet.graphics.Batch() 
    
    self.time = 0.0
    
    if USE_ORDERED_GROUPS:
      self.character_sprite_batch = pyglet.graphics.Batch() 
      self.ordered_groups = []
      for i in range (MAX_ORDERED_GROUPS):
        self.ordered_groups.append(pyglet.graphics.OrderedGroup (i))
    
    self.player = player.Player(Vector3())
    self.player.key_handler = self.key_handler
    self.add_character (self.player)
        
    #self.set_multiline_message ("Get off my lawn!\nOr I'll make you!",10)

  def add_object (self, object):
    self.objects.append (object)
    new_sprite = object.get_sprite()
    self.object_sprites.append (new_sprite)
    new_sprite.batch = self.object_sprite_batch
    object.set_client (self)

  def add_character(self,character):
    self.characters.append (character)
    self.objects.append (character)
    new_sprite = character.get_sprite()

    if USE_ORDERED_GROUPS:
      new_sprite.batch = self.character_sprite_batch
    else:
      character.batch = pyglet.graphics.Batch()
      new_sprite.batch = character.batch
      character.shadow.batch = self.object_sprite_batch

    character.set_client (self)

  def add_particle(self,particle):
    self.add_object(particle)

  def centre_on_player (self):
    self.screen_pos = Vector (self.player.pos.x + self.player.size.x/2 - self.window.width/2,0)
    if self.screen_pos.x < 0:
      self.screen_pos.x = 0
    if self.screen_pos.x > self.world_size.x - self.window.width:
      self.screen_pos.x = self.world_size.x - self.window.width

  def on_key_press(self, button, modifiers):
    self.player.on_key_press(button, modifiers)
    if button == key.P:
      self.paused = not self.paused
    if button == key.ENTER:
      if self.game_over():
        self.director.restart()
        self.start_up()
                
  def on_key_release(self, button, modifiers):
    self.player.on_key_release(button, modifiers)
  
  def draw_healthbar(self, rect, character):
    if character.health == 1:
      period = self.time % 0.2
      if period < 0.1:
        pyglet.gl.glColor3f(1, 0, 0)
      #elif period < 0.2:
      #  pyglet.gl.glColor3f(0, 1, 0)
      else:
        pyglet.gl.glColor3f(0, 0, 0)
      rect.draw_fill()
    else:
      pyglet.gl.glColor3f(0, 0, 0)
      rect.draw_fill()
      
      # Save the right value for later, then change it according to the character's health
      saved_right = rect.right
      total_length = rect.right - rect.left
      ratio_health = character.health / float(character.full_health)
      health_length = ratio_health * total_length
      rect.right = rect.left + health_length
      
      pyglet.gl.glColor3f(1 - ratio_health, ratio_health, 0)
      rect.draw_fill()
      
      # Restore the old value
      rect.right = saved_right
  
  def on_draw (self):
    self.centre_on_player()
    for object in self.objects:
      object.update_sprite(self.screen_pos)
    
    self.background.draw (self.screen_pos)
    #(self.left_fence - self.screen_pos).draw_fill()
    #(self.right_fence - self.screen_pos).draw_fill()    
    
    self.object_sprite_batch.draw()

    self.characters.sort(compare_depth)

    if USE_ORDERED_GROUPS:
      for i,character  in enumerate (self.characters):
        if i >= MAX_ORDERED_GROUPS:
          print "ERROR - too many sprites"
        character.sprite.group = self.ordered_groups[i]
      self.character_sprite_batch.draw()
    else:
      for character  in self.characters:
        character.batch.draw()

    for object in self.objects:
      object.draw2(self.screen_pos)
      
    # Healthbars
    self.draw_healthbar(self.player_healthbar, self.player)
    if self.player.last_enemy_hit != None:
      if self.player.last_enemy_hit.is_dead():
        self.player.last_enemy_hit = None
      else:
        self.draw_healthbar(self.enemy_healthbar, self.player.last_enemy_hit)
    
    for rect in self.fading_rects:
      time = self.fading_rects[rect]
      pyglet.gl.glColor3f(1.0, 1 - time/self.fade_time, 1 - time/self.fade_time)
      (rect - self.screen_pos).draw()
    pyglet.gl.glColor3f(0, 0, 0)

    if (self.message_time_out > 0):
      self.update_message_sprite()
      self.message_sprite.draw()
      self.message_label.draw()
    
    if self.game_over():
      if self.game_over_lose():
        self.game_over_message.draw()
      if self.game_over_win():
        self.victory_message.draw()
      self.restart_message.draw()
  
  def game_over(self):
    return self.game_over_win() or self.game_over_lose()
  def game_over_win(self):
    return self.director.done() and not self.player.is_dead()
  def game_over_lose(self):
    return self.player.is_dead()
    
  def update_message_sprite (self):
    bubble_pos = Vector (self.player.pos.x + -self.screen_pos.x + 100,
                         self.player.pos.y + self.player.pos.z + -self.screen_pos.y + 100)
    self.message_sprite.x = bubble_pos.x
    self.message_sprite.y = bubble_pos.y
    self.message_label.x = bubble_pos.x + SPEECH_AREA_CENTRE_X
    self.message_label.y = bubble_pos.y + SPEECH_AREA_CENTRE_Y
    
  def set_multiline_message (self, text, time_out):
    self.message_label = pyglet.text.Label (text,x=SPEECH_AREA_CENTRE_X,y=SPEECH_AREA_CENTRE_Y, anchor_x="center", anchor_y="center",width=SPEECH_AREA_WIDTH, color = (0,0,0,255),multiline=True,halign="center")
    self.message_time_out = time_out
  
  def set_message (self, text, time_out):
    self.message_label = pyglet.text.Label (text,x=SPEECH_AREA_CENTRE_X,y=SPEECH_AREA_CENTRE_Y, anchor_x="center", anchor_y="center", color = (0,0,0,255),halign="center")
    self.message_time_out = time_out

  def tick(self, delta):
    if self.paused:
      return
      
    self.time += delta
      
    dead_list = []
    for obj in self.objects:
      obj.tick(delta)
      if (obj.is_dead()):
        dead_list.append(obj)
    for dead_obj in dead_list:
      if dead_obj == self.player:
        pass
      dead_obj.destroy()
      self.objects.remove(dead_obj)

    dead_list = []
    for rect in self.fading_rects:
      self.fading_rects[rect] -= delta
      if self.fading_rects[rect] < 0:
        dead_list.append(rect)
    for dead_obj in dead_list:
      del self.fading_rects[dead_obj]
      
    self.director.tick(delta)
    self.message_time_out -= delta
    
  def apply_attack(self, attack, character):
    world_attack_rect = attack.do_attack(character, self.get_near_characters(character.get_centre()))
    world_attack_rect.bottom += character.pos.z
    world_attack_rect.top += character.pos.z
    if config.draw_attack_rects:
      self.fading_rects[world_attack_rect] = self.fade_time

  def get_near_characters (self, point):
    #TODO actually look who's near
    return self.characters

  def add_number_particle (self, number, pos):
    self.add_particle (particle.Number(number,pos,Vector3(0,100,0), self.object_sprite_batch))
