import config
import attack
import images
import character
import pyglet
from pyglet.window import key

class Player(character.Character):
  def __init__(self, pos):
    super(Player, self).__init__(pos)

    self.state.print_state_enabled = config.print_player_state
    self.update_animation()
    
    self.base_move_velocity = 100

    self.show_attack_rects = True
    
    self.team = 0
    
    self.blinking_time = 0.0
       
  def tick(self, time_delta):
    old_state_knocked_down = self.state.is_knocked_down()
    super (Player,self).tick (time_delta)
    # If you just got up
    if old_state_knocked_down and not self.state.is_knocked_down():
      self.blinking_time = 0.6
      
    if self.blinking_time > 0.0:
      if self.blinking_time % 0.2 < 0.1:
        self.sprite.visible = False
      else:
        self.sprite.visible = True

      self.blinking_time -= time_delta
      if self.blinking_time < 0.0:
        self.blinking_time = 0.0
        self.sprite.visible = True

    if self.key_handler[key.UP]:
      self.move_z(self.vertical_speed * time_delta)
    if self.key_handler[key.DOWN]:
      self.move_z(-self.vertical_speed * time_delta)
      
    if self.key_handler[key.LEFT]:
      self.move_x(-self.base_move_velocity * time_delta)
      if self.facing == character.FACING_LEFT:
        self.move_direction()
      else:
        self.move_opposite()
      self.face_left()
    if self.key_handler[key.RIGHT]:
      self.move_x(self.base_move_velocity * time_delta)
      if self.facing == character.FACING_LEFT:
        self.move_opposite()
      else:
        self.move_direction()
      self.face_right()
      
    if self.key_handler[key.E]:
      del self.key_handler[key.E]
      self.high_attack()
    if self.key_handler[key.D]:
      del self.key_handler[key.D]
      self.low_attack()
    # For testing, it's a bad idea to hit yourself
    #if self.key_handler[key.H]:
    #  del self.key_handler[key.H]
    #  self.hit(20)
    if self.key_handler[key.SPACE]:
      del self.key_handler[key.SPACE]
      if self.jump():
        self.vel.y += self.jump_velocity

  # Override the method with the additional blinking check
  def hit(self, damage, vel):
    if self.blinking_time == 0.0:
      super(Player,self).hit(damage, vel)

  def on_key_press(self, button, modifiers):
    pass

  def on_key_release(self, button, modifiers):
    if button == key.LEFT:
      if self.facing == character.FACING_LEFT:
        self.release()
    elif button == key.RIGHT:
      if self.facing == character.FACING_RIGHT:
        self.release()

